const { promisify } = require('util');
const { resolve, join } = require('path');
const { existsSync, readdir, stat, watch:fsw } = require('fs');
const { exec } = require('child_process');

const toExec = promisify(exec);
const toStats = promisify(stat);
const toRead = promisify(readdir);

// modified: lukeed/totalist
async function walk(dir, callback, pre='') {
	await toRead(dir).then(arr => {
		return Promise.all(
			arr.map(str => {
				let abs = join(dir, str);
				return toStats(abs).then(stats => {
					if (!stats.isDirectory()) return;
					callback(join(pre, str), abs, stats);
					return walk(abs, callback, join(pre, str));
				});
			})
		);
	});
}

async function setup(dir, onChange) {
	let output = {};

	try {
		output[dir] = fsw(dir, { recursive: true }, onChange);
	} catch (err) {
		if (err.name !== 'ERR_FEATURE_UNAVAILABLE_ON_PLATFORM') throw err;
		output[dir] = fsw(dir, onChange);
		await walk(dir, (rel, abs) => {
			output[abs] = fsw(abs, onChange);
		});
	}

	return output;
}

async function watch(list, callback, opts={}) {
	const cwd = resolve('.', opts.cwd);
	const dirs = new Set(list.map(str => resolve(cwd, str)).filter(existsSync));
	const ignores = ['node_modules'].concat(opts.ignore || []).map(x => new RegExp(x, 'i'));

	let wip = 0;
	const Triggers = new Set;
	const Watchers = new Map;

	async function handle() {
		await callback();
		if (--wip) return handle();
	}

	// TODO: Catch `EPERM` on Windows for removed dir
	// TODO: `filename` existence still conditional?
	async function onChange(dir, type, filename) {
		if (ignores.some(x => x.test(filename))) return;

		let tmp = join(dir, filename);
		if (Triggers.has(tmp) || !existsSync(tmp)) return;
		if (wip++) return wip = 2;

		let stats = await toStats(tmp);
		if (stats.isDirectory()) return;

		if (opts.clear) console.clear();

		Triggers.add(tmp);
		await handle();
		Triggers.delete(tmp);
	}

	let dir, output, key;
	for (dir of dirs) {
		output = await setup(dir, onChange.bind(0, dir));
		for (key in output) Watchers.set(key, output[key]);
	}
}

async function run() {
	let pid = await toExec.apply(0, arguments);
	if (pid.stdout) process.stdout.write(pid.stdout);
	if (pid.stderr) process.stderr.write(pid.stderr);
}

exports.run = run;
exports.watch = watch;